==> de.duehl.vocabulary.japanese_0.04.63_sources.zip: de.duehl.vocabulary.japanese_0.04.63/de.duehl.vocabulary.japanese/src/de/duehl/vocabulary/japanese/ui/dialog/table/kanji/KanjiTableModel.java <==
package de.duehl.vocabulary.japanese.ui.dialog.table.kanji;

import java.awt.Color;
import java.util.List;

import javax.swing.table.AbstractTableModel;

import de.duehl.basics.datetime.date.ImmutualDate;
import de.duehl.basics.text.Text;
import de.duehl.vocabulary.japanese.common.color.VocableColors;
import de.duehl.vocabulary.japanese.common.data.InternalAdditionalKanjiData;
import de.duehl.vocabulary.japanese.common.persistence.Options;
import de.duehl.vocabulary.japanese.data.symbol.Kanji;
import de.duehl.vocabulary.japanese.logic.symbol.kanji.internal.InternalKanjiDataRequester;
import de.duehl.vocabulary.japanese.ui.dialog.table.TableHelper;

/**
 * Diese Klasse ist ein eigenes Tabellenmodell für die Ansicht der (bislang eingepflegten) Kanji.
 * Es hält die Daten der Tabelle in Form einer Liste von Kanji-Objekten vor und weiß diese
 * anzuzeigen.
 *
 * @version 1.01     2024-11-13
 * @author Christian Dühl
 */

public class KanjiTableModel extends AbstractTableModel {

    private static final int NUMBER_OF_COLUMNS = 17;

    private static final long serialVersionUID = 1L;


    /** Die Programmoptionen. */
    private final Options options;

    /** Die Liste mit den Kanji der Tabelle. */
    private final List<Kanji> kanjiList;

    /** Das Objekt das zu einem Kanji die internen, benutzerabhängigen Daten abrufen kann. */
    private final InternalKanjiDataRequester requester;

    /**
     * Konstruktor.
     *
     * @param options
     *            Die Programmoptionen.
     * @param kanjiList
     *            Die Liste mit den Kanji der Tabelle.
     * @param requester
     *            Das Objekt das zu einem Kanji die internen, benutzerabhängigen Daten abrufen kann.
     */
    public KanjiTableModel(Options options, List<Kanji> kanjiList,
            InternalKanjiDataRequester requester) {
        super();
        this.options = options;
        this.kanjiList = kanjiList;
        this.requester = requester;
    }

    /** Ermittelt die Zeilenzahl. */
    @Override
    public int getRowCount() {
        return kanjiList.size();
    }

    /** Ermittelt die Spaltenzahl. */
    @Override
    public int getColumnCount() {
        return NUMBER_OF_COLUMNS;
    }

    /**
     * Gibt an, ob die Zelle bearbeitbar ist. Dies muss für die Buttons der Fall sein.
     *
     * @param rowIndex
     *            Zeilenindex (die erste hat die 0)
     * @param columnIndex
     *            Spaltenindex (die erste hat die 0)
     */
    @Override
    public boolean isCellEditable(int rowIndex, int columnIndex) {
        return columnIndex == 9 || columnIndex == 16;
    }

    /**
     * Ermittelt den Tabelleninhalt der angegebenen Zelle.
     *
     * Die Tabelle enthält die folgenden Spalten:
     *     - Die laufende Nummer.
     *     - Das Kanji in UTF-8 Darstellung.
     *     - Die deutsche Beschreibung.
     *     - Das Wort in Kanji und Hiragana in der häufigsten Bedeutung.
     *     - Das Wort in Hiragana in der häufigsten Bedeutung.
     *     - Die Nummer des Kanji im Buch "Kanji und Kana" von Wolfgang Hadamitzky.
     *     - Die Seite auf der das Kanji im Buch "Kanji und Kana" von W. Hadamitzky zu finden ist.
     *     - Die Liste der sinojapanischen ON-Lesungen des Kanji (groß geschrieben).
     *     - Die Liste der rein japanischen kun-Lesungen des Kanji (klein geschrieben).
     *     - Button zum Betrachten der aller Vokabeln, in denen das Kanji vorkommt.
     * Außerdem die folgenden Spalten aus den internen Kanji-Daten:
     *     - Das Datum an dem das Kanji das erste Mal gesehen wurde.
     *     - Die Anzahl, wie oft das Kanji getestet wurde.
     *     - Die Anzahl, wie oft das Kanji erfolgreich getestet wurde.
     *     - Das Datum des letzten Tests.
     *     - Das Datum des letzten erfolgreichen Tests.
     *     - Das Abschneiden bei den letzten Tests.
     *     - Button zum Betrachten der Kanji mit allen internen Daten
     *
     * @param rowIndex
     *            Zeilenindex (die erste hat die 0)
     * @param columnIndex
     *            Spaltenindex (die erste hat die 0)
     */
    @Override
    public Object getValueAt(int rowIndex, int columnIndex) {
        Kanji kanji = kanjiList.get(rowIndex);
        InternalAdditionalKanjiData data = requester.getInternalDataForKanji(kanji);
        switch (columnIndex) {
            case 0:
                return rowIndex + 1;
            case 1:
                return kanji.getCharacter();
            case 2:
                return kanji.getGermanMeaning();
            case 3:
                return kanji.getNormalKanjiHiraganaWord();
            case 4:
                return kanji.getNormalHiraganaWord();
            case 5:
                return kanji.getNumberInKanjiAndKanaBook();
            case 6:
                return kanji.getPageNumberInKanjiAndKanaBook();
            case 7:
                return Text.join(", ", kanji.getOnLesungen());
            case 8:
                return Text.join(", ", kanji.getKunLesungen());
            case 9:
                return "Vokabeln mit dem Kanji anzeigen";
            case 10:
                return data.getFirstSeenDate();
            case 11:
                return data.getTestCount();
            case 12:
                return data.getCorrectTestCount();
            case 13:
                return TableHelper.dateOrEmpty(data.getLastTestDate(), data.getTestCount());
            case 14:
                return TableHelper.dateOrEmpty(data.getLastCorrectTestDate(),
                        data.getCorrectTestCount());
            case 15:
                return TableHelper.createMonospaceHtml(data.getLastTenTestResultsAsStorageString());
            case 16:
                return "Details";
            default:
                throw new RuntimeException("Unzuläsiger Spaltenindex '" + columnIndex + "'.");
        }
    }

    @Override
    public Class<?> getColumnClass(int columnIndex) {
        switch (columnIndex) {
            case 0:
            case 5:
            case 6:
            case 11:
            case 12:
                return Integer.class;
            case 1:
            case 2:
            case 3:
            case 4:
            case 7:
            case 8:
            case 15:
                return String.class;
            case 10:
            case 13:
            case 14:
                return ImmutualDate.class;
                //  9 ist der Button zum Anzeigen der zugehörigen Vokabeln
                // 16 ist der Button zum Anzeigen der Details des Kanji
            default:
                return String.class;
        }
    }

    /**
     * Erzeugt die Vordergrundfarbe für die Zeile mit dem übergebenen Zeilenindex.
     *
     * @param rowIndex
     *            Zeilenindex (die erste hat die 0)
     * @return Farbwert für den Vordergrund.
     */
    public Color determineRowForegroundColor(int rowIndex) {
        Kanji kanji = kanjiList.get(rowIndex);
        InternalAdditionalKanjiData data = requester.getInternalDataForKanji(kanji);
        VocableColors vocableColors = new VocableColors(options);
        return vocableColors.determineKanjiForegroundColor(data, true);
    }

    /**
     * Erzeugt die Hintergrundfarbe für die Zeile mit dem übergebenen Zeilenindex.
     *
     * @param rowIndex
     *            Zeilenindex (die erste hat die 0)
     * @return Farbwert für den Hintergrund.
     */
    public Color determineRowBackgroundColor(int rowIndex) {
        Kanji kanji = kanjiList.get(rowIndex);
        InternalAdditionalKanjiData data = requester.getInternalDataForKanji(kanji);
        VocableColors vocableColors = new VocableColors(options);
        return vocableColors.determineKanjiBackgroundColor(data, true);
    }

}

==> de.duehl.vocabulary.japanese_0.05.00_sources.zip: de.duehl.vocabulary.japanese_0.05.00/de.duehl.vocabulary.japanese/src/de/duehl/vocabulary/japanese/ui/dialog/table/kanji/KanjiTableModel.java <==
package de.duehl.vocabulary.japanese.ui.dialog.table.kanji;

import java.awt.Color;
import java.util.List;

import javax.swing.table.AbstractTableModel;

import de.duehl.basics.datetime.date.ImmutualDate;
import de.duehl.basics.text.Text;
import de.duehl.vocabulary.japanese.common.color.VocableColors;
import de.duehl.vocabulary.japanese.common.data.InternalAdditionalKanjiData;
import de.duehl.vocabulary.japanese.common.persistence.Options;
import de.duehl.vocabulary.japanese.data.FumikoDataStructures;
import de.duehl.vocabulary.japanese.data.symbol.Kanji;
import de.duehl.vocabulary.japanese.logic.symbol.kanji.internal.InternalKanjiDataRequester;
import de.duehl.vocabulary.japanese.ui.dialog.table.TableHelper;

/**
 * Diese Klasse ist ein eigenes Tabellenmodell für die Ansicht der (bislang eingepflegten) Kanji.
 * Es hält die Daten der Tabelle in Form einer Liste von Kanji-Objekten vor und weiß diese
 * anzuzeigen.
 *
 * @version 1.01     2025-11-20
 * @author Christian Dühl
 */

public class KanjiTableModel extends AbstractTableModel {

    private static final int NUMBER_OF_COLUMNS = 17;

    private static final long serialVersionUID = 1L;


    /** Die Liste mit den Kanji der Tabelle. */
    private final List<Kanji> kanjiList;

    /** Die Programmoptionen. */
    private final Options options;

    /** Das Objekt das zu einem Kanji die internen, benutzerabhängigen Daten abrufen kann. */
    private final InternalKanjiDataRequester requester;

    /**
     * Konstruktor.
     *
     * @param kanjiList
     *            Die Liste mit den Kanji der Tabelle.
     * @param dataStructures
     *            Die Datenstrukturen des Vokabeltrainers.
     */
    public KanjiTableModel(List<Kanji> kanjiList,
            FumikoDataStructures dataStructures) {
        super();

        this.kanjiList = kanjiList;
        this.options = dataStructures.getOptions();
        this.requester = dataStructures.getInternalKanjiDataRequester();
    }

    /** Ermittelt die Zeilenzahl. */
    @Override
    public int getRowCount() {
        return kanjiList.size();
    }

    /** Ermittelt die Spaltenzahl. */
    @Override
    public int getColumnCount() {
        return NUMBER_OF_COLUMNS;
    }

    /**
     * Gibt an, ob die Zelle bearbeitbar ist. Dies muss für die Buttons der Fall sein.
     *
     * @param rowIndex
     *            Zeilenindex (die erste hat die 0)
     * @param columnIndex
     *            Spaltenindex (die erste hat die 0)
     */
    @Override
    public boolean isCellEditable(int rowIndex, int columnIndex) {
        return columnIndex == 9 || columnIndex == 16;
    }

    /**
     * Ermittelt den Tabelleninhalt der angegebenen Zelle.
     *
     * Die Tabelle enthält die folgenden Spalten:
     *     - Die laufende Nummer.
     *     - Das Kanji in UTF-8 Darstellung.
     *     - Die deutsche Beschreibung.
     *     - Das Wort in Kanji und Hiragana in der häufigsten Bedeutung.
     *     - Das Wort in Hiragana in der häufigsten Bedeutung.
     *     - Die Nummer des Kanji im Buch "Kanji und Kana" von Wolfgang Hadamitzky.
     *     - Die Seite auf der das Kanji im Buch "Kanji und Kana" von W. Hadamitzky zu finden ist.
     *     - Die Liste der sinojapanischen ON-Lesungen des Kanji (groß geschrieben).
     *     - Die Liste der rein japanischen kun-Lesungen des Kanji (klein geschrieben).
     *     - Button zum Betrachten der aller Vokabeln, in denen das Kanji vorkommt.
     * Außerdem die folgenden Spalten aus den internen Kanji-Daten:
     *     - Das Datum an dem das Kanji das erste Mal gesehen wurde.
     *     - Die Anzahl, wie oft das Kanji getestet wurde.
     *     - Die Anzahl, wie oft das Kanji erfolgreich getestet wurde.
     *     - Das Datum des letzten Tests.
     *     - Das Datum des letzten erfolgreichen Tests.
     *     - Das Abschneiden bei den letzten Tests.
     *     - Button zum Betrachten der Kanji mit allen internen Daten
     *
     * @param rowIndex
     *            Zeilenindex (die erste hat die 0)
     * @param columnIndex
     *            Spaltenindex (die erste hat die 0)
     */
    @Override
    public Object getValueAt(int rowIndex, int columnIndex) {
        Kanji kanji = kanjiList.get(rowIndex);
        InternalAdditionalKanjiData data = requester.getInternalDataForKanji(kanji);
        switch (columnIndex) {
            case 0:
                return rowIndex + 1;
            case 1:
                return kanji.getCharacter();
            case 2:
                return kanji.getGermanMeaning();
            case 3:
                return kanji.getNormalKanjiHiraganaWord();
            case 4:
                return kanji.getNormalHiraganaWord();
            case 5:
                return kanji.getNumberInKanjiAndKanaBook();
            case 6:
                return kanji.getPageNumberInKanjiAndKanaBook();
            case 7:
                return Text.join(", ", kanji.getOnLesungen());
            case 8:
                return Text.join(", ", kanji.getKunLesungen());
            case 9:
                return "Vokabeln mit dem Kanji anzeigen";
            case 10:
                return data.getFirstSeenDate();
            case 11:
                return data.getTestCount();
            case 12:
                return data.getCorrectTestCount();
            case 13:
                return TableHelper.dateOrEmpty(data.getLastTestDate(), data.getTestCount());
            case 14:
                return TableHelper.dateOrEmpty(data.getLastCorrectTestDate(),
                        data.getCorrectTestCount());
            case 15:
                return TableHelper.createMonospaceHtml(data.getLastTenTestResultsAsStorageString());
            case 16:
                return "Details";
            default:
                throw new RuntimeException("Unzuläsiger Spaltenindex '" + columnIndex + "'.");
        }
    }

    @Override
    public Class<?> getColumnClass(int columnIndex) {
        switch (columnIndex) {
            case 0:
            case 5:
            case 6:
            case 11:
            case 12:
                return Integer.class;
            case 1:
            case 2:
            case 3:
            case 4:
            case 7:
            case 8:
            case 15:
                return String.class;
            case 10:
            case 13:
            case 14:
                return ImmutualDate.class;
                //  9 ist der Button zum Anzeigen der zugehörigen Vokabeln
                // 16 ist der Button zum Anzeigen der Details des Kanji
            default:
                return String.class;
        }
    }

    /**
     * Erzeugt die Vordergrundfarbe für die Zeile mit dem übergebenen Zeilenindex.
     *
     * @param rowIndex
     *            Zeilenindex (die erste hat die 0)
     * @return Farbwert für den Vordergrund.
     */
    public Color determineRowForegroundColor(int rowIndex) {
        Kanji kanji = kanjiList.get(rowIndex);
        InternalAdditionalKanjiData data = requester.getInternalDataForKanji(kanji);
        VocableColors vocableColors = new VocableColors(options);
        return vocableColors.determineKanjiForegroundColor(data, true);
    }

    /**
     * Erzeugt die Hintergrundfarbe für die Zeile mit dem übergebenen Zeilenindex.
     *
     * @param rowIndex
     *            Zeilenindex (die erste hat die 0)
     * @return Farbwert für den Hintergrund.
     */
    public Color determineRowBackgroundColor(int rowIndex) {
        Kanji kanji = kanjiList.get(rowIndex);
        InternalAdditionalKanjiData data = requester.getInternalDataForKanji(kanji);
        VocableColors vocableColors = new VocableColors(options);
        return vocableColors.determineKanjiBackgroundColor(data, true);
    }

}
